\ Lexical scanner generator - save lex tables to file

\ Copyright (C) Gerry Jackson 2006

\ This software is free; you can redistribute it and/or modify it in
\ any way provided you acknowledge the original source and copyright

\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
\ ------------------------------------------------------------------

.( SaveTables.fth loading ...) cr

\ ------------------------------------------------------------------------
\ Output file handling

variable lexFile 0 lexFile !				\ file identifier

: createFile	( caddr u -- )
	w/o create-file				( -- fid ior )
	abort" Unable to create output file"
	lexFile !
;

: closeFile		( -- )
	lexFile @ close-file					( -- ior )
	abort" Unable to close output file"
;

\ ------------------------------------------------------------------------
\ Writing to output file

: writeText ( caddr u -- )
   lexFile @ write-file abort" Writing text to file failed"
;

: writeLine ( caddr u -- )
   lexFile @ write-line abort" Writing a line of text to file failed"
;

: writeNewLine    ( -- ) 0 0 writeLine ;

: writeSpace		( -- ) s"  " writeText ;

: writeSpaces     ( n -- ) 0 ?do writeSpace loop ;

: writeNumber.R   ( n u -- )
   over 2>r abs 0 <# #s r> sign #>
   r> over - 0 max writeSpaces writeText
;

: writeNumber		( n -- ) 0 writeNumber.R ;

\ -----------------------------------------------------------------------------

: underline    ( n -- )
   s" \ " writeText
   2 - dup 0>
   if
      0
      do
         s" -" writeText
      loop
   else
      drop
   then
   writeNewLine
;

: saveBaseDefTokData
   s" 0 value BaseDefaultData" writeLine
   s" here to BaseDefaultData" writeLine writeNewLine
   s" \ State    Base   Default  Token" dup >r writeLine
   r@ underline
   #states @ 0
   ?do
      s"  ( " writeText i 3 writeNumber.R s"  ) " writeText
      i lexDefBase lex_base    @ 5 writeNumber.R s"  ,"  writeText
      i lexDefBase lex_default @ 6 writeNumber.R s"  , " writeText
      i lex_tokens @ 5 writeNumber.R s"  ," writeText
      writeNewLine
   loop
   r> underline
;

: saveCheck&NextData
   NextCheck nc_size @ writeNumber s"  value maxCheck" writeLine
   s" 0 value CheckNextData" writeLine
   s" here to CheckNextData" writeLine writeNewLine
   s" \ Index    Check    Next" dup >r writeLine
   r@ underline
   NextCheck nc_size @ 0
   ?do
      s"  ( " writeText i 3 writeNumber.R s"  ) " writeText
      i lexNextCheck lex_check @ 5 writeNumber.R s"  ,"  writeText
      i lexNextCheck lex_next  @ 6 writeNumber.R s"  , " writeText
      writeNewLine
   loop
   r> underline
;

: save#states
  #states @ writeNumber writeSpace
  s" value #states" writeLine
  writeNewLine
;

\ Set the default output file name, may be redefined by the user
\ calling setOutputFile. 

variable outputFile

:noname ( -- caddr u ) s" lexgen\lextables.fth" ; outputFile !

\ The use of 2>r and 2r> in setOutputFile is for those systems, like GForth,
\ that use the data stack for the control stack

: setOutputFile   ( caddr u -- )
   2>r :noname 2r> postpone sliteral postpone ; ( -- xt )
   outputFile !
;

: saveAllTables
   outputFile @ execute createFile
   s" \ Tables for a lexical scanner " writeText
   s" - generated by LexGen" writeLine writeNewLine
   s" decimal" writeLine writeNewLine
   save#states
   saveBaseDefTokData
   writeNewLine
   saveCheck&NextData
   closeFile
;

\ -----------------------------------------------------------------------------

.( SaveTables.fth loaded. ) .s

